@@ -1,6 +1,7 @@
 import math
 import re
 import pyproj
+import warnings
 
 '''
 Methods for representing geographic coordinates (latitude and longitude)
@@ -331,23 +332,50 @@ class LatLon:
     def __idiv__(self, other):
         # other is a scalar
         return self.__div__(other)
-        
-    def distance(self, other, ellipse = 'WGS84'):
+    
+    def heading_initial(self, other, ellipse = 'WGS84'):
         '''
-        Returns great circle distance between two lat/lon coordinates in km
-        using the WGS84 ellipsoid
+        Returns initial bearing between two LatLon objects in degrees using pyproj.
+        Assumes the WGS84 ellipsoid by default. Choose ellipse = 'sphere'
+        for the FAI ellipsoid.
         '''
         lat1, lon1 = self.lat.decimal_degree, self.lon.decimal_degree
         lat2, lon2 = other.lat.decimal_degree, other.lon.decimal_degree
         g = pyproj.Geod(ellps = ellipse)
-        return g.inv(lon1, lat1, lon2, lat2)[2]/1000.0
+        return g.inv(lon1, lat1, lon2, lat2, radians = False)[0]
+    
+    def heading_reverse(self, other, ellipse = 'WGS84'):
+        '''
+        Returns reverse bearing between two LatLon objects in degrees using pyproj.
+        Assumes the WGS84 ellipsoid by default. Choose ellipse = 'sphere'
+        for the FAI ellipsoid.
+        '''
+        lat1, lon1 = self.lat.decimal_degree, self.lon.decimal_degree
+        lat2, lon2 = other.lat.decimal_degree, other.lon.decimal_degree
+        g = pyproj.Geod(ellps = ellipse)
+        return g.inv(lon1, lat1, lon2, lat2, radians = False)[1]
+        
+    def distance(self, other, ellipse = 'WGS84'):
+        '''
+        Returns great circle distance between two LatLon objects in km using pyproj.
+        Assumes the WGS84 ellipsoid by default. Choose ellipse = 'sphere'
+        for the FAI ellipsoid.
+        '''
+        lat1, lon1 = self.lat.decimal_degree, self.lon.decimal_degree
+        lat2, lon2 = other.lat.decimal_degree, other.lon.decimal_degree
+        g = pyproj.Geod(ellps = ellipse)
+        return g.inv(lon1, lat1, lon2, lat2, radians = False)[2]/1000.0
         
     def distance_sphere(self, other, radius = 6371.0):
         '''
+        -- Deprecated in v0.70. Use distance(other, ellipse = 'sphere') instead --
+        
         Returns great circle distance between two lat/lon coordinates on a sphere
         using the Haversine formula. The default radius corresponds to the FAI sphere
         with units in km.
         '''
+        warnings.warn("Deprecated in v0.70. Use distance(other, ellipse = 'sphere') instead",
+                      DeprecationWarning)
         lat1, lon1 = self.lat.decimal_degree, self.lon.decimal_degree
         lat2, lon2 = other.lat.decimal_degree, other.lon.decimal_degree
         pi = math.pi/180.
@@ -362,6 +390,17 @@ class LatLon:
         arc = math.acos(cos)
         return arc*radius
     
+    def offset(self, heading_initial, distance, ellipse = 'WGS84'):
+        '''
+        Offset a LatLon object by a heading (in degrees) and distance (in km) 
+        to return a new LatLon object
+        '''
+        lat1, lon1 = self.lat.decimal_degree, self.lon.decimal_degree
+        g = pyproj.Geod(ellps = ellipse)
+        distance = distance * 1000 # Convert km to meters
+        lon2, lat2, back_bearing = g.fwd(lon1, lat1, heading_initial, distance, radians = False)
+        return LatLon(Latitude(lat2), Longitude(lon2))
+    
     def to_string(self, formatter = 'D'):
         '''
         Return string representation of lat and lon as a 2-element tuple
@@ -394,8 +433,13 @@ def test():
     palmyra = string2latlon('N 5, 52.998', 'W 162, 4.998', 'H% %d%, %M') # An alternative complex string
     print palmyra.to_string('D') # Print coordinate to decimal degrees (returns ('5.8833', '-162.0833'))
     honolulu = LatLon(Latitude(21.3), Longitude(-157.8167))
-    print palmyra.distance(honolulu) # WGS84 distance is 1766.69130376 km
-    print palmyra.distance_sphere(honolulu) # FAI distance is 1774.77188181 km
+    print palmyra.distance(honolulu, ellipse = 'sphere') # FAI distance is 1774.77188181 km
+    distance = palmyra.distance(honolulu) # WGS84 distance is 1766.69130376 km
+    print distance
+    initial_heading = palmyra.heading_initial(honolulu) # Initial heading to Honolulu on WGS84 ellipsoid
+    print initial_heading
+    hnl = palmyra.offset(initial_heading, distance) # Reconstruct lat/lon for Honolulu based on offset from Palmyra
+    print hnl.to_string('D') # Coordinates of Honolulu are latitude 21.3, longitude -157.8167
 
 if __name__ == '__main__':
     test()
